#________________________________
# internal function Hill_inverse
#________________________________

Hill_inverse  <- function(ytab, k) {
  n <- length(ytab)
  ysort <- sort(ytab)   
  g1 <- ysort[n-k]
  som1 <- 0
  # result
  sum(log(ysort[(n-(k-1)):n]) - log(g1))/k
}

#__________________________________
# Weissman quantile estimator: 
#__________________________________

Weissman_quantile <- function(ytab, pn, k) {
  #inputs:
  #       ytab = sample of nx1 obs
  #        pn  = order < 1/n
  #         k  = sample fraction varying from "1" up to "n-1"
  #output: res = Weissman estimator of the extreme (1-pn)th quantile.
  n <- length(ytab);
  ysort <- sort(ytab)   
  gam <- Hill_inverse(ytab, k)
  ysort[n-k]*(k/(n*pn))^(gam)
}

#__________________________________
# Plug-in extremile estimator based on the asymptotic connection:         
#__________________________________

Plugin_extremile <- function(ytab, pn, k)  {
  #inputs:
  #       ytab = sample of nx1 obs
  #        pn  = order < 1/n
  #         k  = sample fraction varying from "1" up to "n-1"       
  #output: res = Plug-in estimator of the extreme (1-pn)th extremile $\widehat\xi_{1-p_n}^{\star}$.
  q <- Weissman_quantile(ytab, pn, k)
  s <- Hill_inverse(ytab, k)
  s <- s * (s < 1)
  Gs <- gamma(1-s) * (log(2))^s
  q * Gs
}

#__________________________________
### internal functions
#__________________________________

.sg <- function(gam) {
  #input: gam = extremile order in (0,1)
  #output: power $s(gam)$ in the definition of the measure $K_{gam}$
  log(1/2)/log(1-gam)
}

# _________________________________
.Kg <- function(gam, t) {
  #input: gam = extremile order in (0,1)
  #         t = real number in the support [0,1] of the $K_{gam}$
  #output: measure $K_{gam}(t)$ 
  ((1-(1-t)^(.sg(gam)))*(0<gam)*(gam<=0.5))+((t^(.sg(1-gam)))*(0.5<gam)*(gam<1))
}

# _________________________________
.Jg <- function(gam, t) {
  #input: gam = extremile order in (0,1)
  #         t = real number in the support (0,1] of the density $J_{gam}=K'_{gam}$
  #output: weight-generating function $J_{gam}(t)$ 
  if ((0<gam) & (gam<=0.5))
    res <- (.sg(gam))*(1-t)^(.sg(gam)-1) 
  
  if ((0.5<gam) & (gam<1)) 
    res <- (.sg(1-gam))*t^(.sg(1-gam)-1)
  
  res
}


###  main function

als_extremile <- function(ytab, probs) {
  #inputs:
  #       ytab = sample of nx1 obs
  #        gam = order in (0,1)     
  #output: res = Ordinary $gam$th sample ALS estimator $\widetilde\xi_{gam}$.
  #$$ \widetilde{\xi}_{\gamma} = \frac{1}{n}  \sum_{i=1}^n  J_{gam}(i/n) * Ysort[i] $$
  #__________________________________
  
  sytab <- sort(ytab)
  n <- length(ytab)
  num <- 0
  den <- 0
  
  for (i in 1:n) {
    num <- num + (.Jg(probs, i/n)*sytab[i])
    den <- den + (.Jg(probs, i/n))    
  }
  
  num/den
}    

#__________________________________

# internal function

#__________________________________

.f1<-function(x, gam) {
  cste <- ifelse(gam <= 0.5, 1 - gam, gam)
  
  if (gam <= 0.5)
    (log(1/2)/log(cste))*(1-x)^((log(1/2)/log(cste))-1)
  else
    (log(1/2)/log(cste))*x^((log(1/2)/log(cste))-1)
}

#__________________________________
# True extremile of the Frechet distribution:           
#__________________________________

xfrechet <- function(gam, loc = 0, scale = 1, shape) {
  #inputs:
  #        gam   = extremile order in (0,1)
  #        loc, scale, shape = the location parameter a, scale parameter b, and shape parameter s. 
  #output: gives the theoretical (gam)th extremile $\xi_{gam}$   
  
  #__________________________________                 
  
  compute <- function(gam, loc, scale, shape) {
    
    f <- function(x, loc, scale, shape, gam) {
      VGAM::qfrechet(x, loc, scale, shape)*.f1(x,gam)
    }
    
    integrate(f, lower = 0, upper = 1, loc = loc, scale = scale, shape = shape, 
              gam = gam, rel.tol = 5*10^8*.Machine$double.eps)$value
  }
  
  gam[gam <= 0 | gam >= 1] <- NA
  
  res<-rep(NA, length(gam))
  
  res[!is.na(gam)] <- unlist(lapply(na.omit(gam), compute, loc, scale, shape))
  
  return(res)
}

#__________________________________

# True extremile of the Pareto distribution:                  

#__________________________________

xpareto <- function(gam, shape, scale) {
  #inputs:
  #        gam   = extremile order in (0,1)
  #        shape, scale = parameters. Must be strictly positive. 
  #output: theoretical (gam)th extremile $\xi_{gam}$   
  #__________________________________                 
  
  compute <- function(gam, shape, scale) {
    
    f <- function(x, shape, scale, gam) {
      actuar::qpareto(x, shape, scale)*.f1(x,gam)
    }
    
    integrate(f, lower = 0, upper = 1, shape = shape, 
              scale = scale, gam = gam, rel.tol = 5*10^8*.Machine$double.eps)$value
  }
  
  gam[gam <= 0 | gam >= 1] <- NA
  
  res <- rep(NA, length(gam))
  
  res[!is.na(gam)] <- unlist(lapply(na.omit(gam), compute, shape, scale))
  
  return(res)
}

#__________________________________
# True extremile of the Student t(df) distribution:         
#__________________________________

xt <- function(gam, df, ncp = 0) {
  #inputs:
  #        gam = extremile order in (0,1]
  #        df  = Student's degree of freedom
  #output: theoretical (gam)th extremile $\xi_{gam}$   
  #__________________________________                         
  
  compute <- function(gam, df, ncp) {
    
    f <- function(x, df, ncp, gam) {
      qt(x, df, ncp)*.f1(x,gam)
    }
    
    integrate(f, lower = 0, upper = 1, df = df, ncp = ncp, 
              gam = gam, rel.tol = 5*10^8*.Machine$double.eps)$value
  }
  
  gam[gam <= 0 | gam >= 1] <- NA
  
  res <- rep(NA, length(gam))
  
  res[!is.na(gam)] <- unlist(lapply(na.omit(gam), compute, df, ncp))
  
  return(res)
}
