#-------------------------------------------------------------------------#
#
#         Extremiles: A new perspective on asymmetric least squares 
#         
#                     Updated: November 10, 2017 
#  
#-------------------------------------------------------------------------#

# Content:  1) main code, line 15 
#           2) figures 1-3, line 163



##########################################################################
#                   (1)  Main code
##########################################################################
# Packages required:
# install.packages(c("VGAM", "snowfall"))

require("actuar")
require("VGAM")
require("snowfall")


# Sample size:
n <- 1000

# number of core
# depends on your machine
# If you want to know the number of available core : 
# library("parallel")
# detectCores() 

nb.core <- 40

# Nb of simulated samples: 
B <- 15000 #10000


#____________________________________________________________________
#
# function which computes the estimates for 1b (b=1 to B) :
#
#____________________________________________________________________

calcPar <- function(b, type, n, gam) {
  # arguments : 
  # - b = 1 to B
  # - type = "frechet", "pareto" or "student"
  # - n = number of observations in the sample
  # - gam = 1/4,  1/3 or 1/2.5

  taunp <- 1 - 1/n

  #grid of values of "k":
  kl <- floor(log(n^0.9)) + 1       
  ku <- floor(n/log(n^0.9))
  kk <- kl:ku 
  nk <- length(kk)
  
  # initialisation
  hat <- numeric(nk)     #Indirect extremile
  tilde <- numeric(nk)   #Direct extremile
  
  # simulations 
  
  ytab <- switch(type, frechet = VGAM::rfrechet(n, location = 0, scale = 1, shape = 1/gam), # Frechet(alpha)  
                       pareto  = actuar::rpareto(n, scale = 1, shape = 1/gam),  # Pareto(alpha=1/gamma)  
                       student = rt(n, (1/gam))   # Student(df=1/gamma)  
                 
  )
  
  
  # Computation of the estimates:                          
  
  for (i in 1:nk) { 
    k <- kk[i] 
    #Hill's estimate:
    gamH <- Hill_inverse(ytab, k)
    #extremile level tau'_n = 1-pn:
    pn <- 1/n
    #Indirect extremile $\widehat\xi_{\tau'_n}^{Q,\star}$:
    hat[i] <- Plugin_extremile(ytab, pn, k)
    #Direct extremile $\widehat{\xi}_{\tau'_n}^{M,\star}$:
    tilde[i] <- (n*pn/k)^(-gamH) * als_extremile(ytab, probs = 1-(k/n))                                                                
  }
  
  return(list(hat = hat, tilde = tilde))
}



# Applications :
# we apply the function calcPar to different values of "type" and "gam"
path <- "/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123"
source(paste(path, "/internal_functions.R", sep = ""))


for (type in c("frechet", "pareto", "student")) {
  for (gam in c(1/4, 1/3, 2/5)) {
    # we execute each simulation on a core 
    sfInit(parallel = TRUE, cpus = nb.core)
    sfSource(paste(path, "/internal_functions.R", sep = ""))
    time.slot <- system.time(
      res <- sfClusterApplyLB(1:B, calcPar, type = type, n = n, gam = gam) 
    )
    sfStop()
    # the time for one value of type and one value of gam 
    print(time.slot)
    # depends on the the type argument : 
    # grid of values of "k":
    kl <- floor(log(n^0.9)) + 1       
    ku <- floor(n/log(n^0.9))
    kk <- kl:ku 
    nk <- length(kk)
    
    # we store the results in a matrix (could be avoided by doing a sum on the vectors of results)
    hat <- matrix(0, B, nk)
    tilde <- matrix(0, B, nk)    
    
    for (b in 1:B) {
      hat[b, ] <- res[[b]]$hat
      tilde[b, ] <- res[[b]]$tilde
    }
    
    #--------------------------------------------------------------   
    #True extremile $\xi_{1-1/n}$:

    if (type %in% c("frechet")) {
      tVaR <- xfrechet(1 - 1/n, loc = 0, scale = 1, shape = 1/gam)
    }
    
    if (type %in% c("pareto")) {
      tVaR <- xpareto(1 - 1/n, shape = 1/gam, scale = 1)
    }
    
    if (type %in% c("student")) {
      tVaR <- xt(1 - 1/n, df = 1/gam, ncp = 0)
    }
    
    #--------------------------------------------------------------   
    # compute the results
    
    hat_MSE <- colMeans((tVaR - hat)^2, na.rm = TRUE) #Indirect (plug-in) Xpectile estimator
    tilde_MSE <- colMeans((tVaR - tilde)^2)    # Direct (LAWS) Xpectile estimator    
    hat_Bias <- colMeans(hat - tVaR, na.rm = TRUE)      
    tilde_Bias <- colMeans(tilde - tVaR)        
    
    xt_als_NMSE <- tilde_MSE/(tVaR)^2   
    xt_NMSE = hat_MSE/(tVaR)^2   
    xt_NBias = hat_Bias/tVaR
    xt_als_NBias = tilde_Bias/tVaR    
    
    # save the results in the appropriate file  
#    save(kk,  tilde_Bias,  hat_Bias, xt_als_NBias, xt_NBias,  tilde_MSE,  hat_MSE, xt_als_NMSE,  xt_NMSE, 
#         file = paste(type, "_gam", 1/gam, "_xtremiles.RData", sep = ""))
  }
  
}


##########################################################################
#                   (2)  Figures 1-3
##########################################################################

install.packages("ggplot2")
require(ggplot2)
#
library(reshape2)

#---------------------------------------------------------------------------------------------------
#
#    Figure 1 :   
#                            
#---------------------------------------------------------------------------------------------------

  dev.new()

require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,3)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}


  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//frechet_gam4_xtremiles.RData")
    
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NMSE)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NMSE) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
#scale_y_continuous("MSE") +
scale_y_continuous("MSE", limit = c(0.016,0.13)) + # rho=2.5
#scale_y_continuous("MSE", limit = c(0.19,0.53)) + # rho=3
scale_x_continuous("k")+ # rho=4
labs(title = "Fréchet")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1))
   
  
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//pareto_gam4_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NMSE)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NMSE) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("MSE", limit = c(0.1,2.75)) + # rho=2.5
#scale_y_continuous("MSE") +
scale_x_continuous("k")+
labs(title = "Pareto")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,2))

    
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//student_gam4_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NMSE)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NMSE) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("MSE", limit = c(0.1,2.75)) + # rho=2.5
#scale_y_continuous("MSE") +
scale_x_continuous("k")+
labs(title = "Student")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,3))

   
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//frechet_gam4_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NBias)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NBias) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Fréchet")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,1))


  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//pareto_gam4_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NBias)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NBias) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Pareto")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,2))

  
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//student_gam4_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NBias)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NBias) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Student")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,3))




#---------------------------------------------------------------------------------------------------
#
#   Figure 2 :   
#                            
#---------------------------------------------------------------------------------------------------

  dev.new()

#require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,3)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}


  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//frechet_gam3_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NMSE)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NMSE) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
#scale_y_continuous("MSE") +
scale_y_continuous("MSE", limit = c(0.03,0.4)) + # rho=2.5
#scale_y_continuous("MSE", limit = c(0.19,0.53)) + # rho=3
scale_x_continuous("k")+ # rho=4
labs(title = "Fréchet")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1))
   
  
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//pareto_gam3_xtremiles.RData")
      
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NMSE)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NMSE) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("MSE", limit = c(0.25,1.5)) + # rho=2.5
#scale_y_continuous("MSE") +
scale_x_continuous("k")+
labs(title = "Pareto")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,2))

    
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//student_gam3_xtremiles.RData")  
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NMSE)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NMSE) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("MSE", limit = c(0.15,3)) + # rho=2.5
#scale_y_continuous("MSE") +
scale_x_continuous("k")+
labs(title = "Student")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,3))

   
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//frechet_gam3_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NBias)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NBias) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Fréchet")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,1))


  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//pareto_gam3_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NBias)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NBias) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Pareto")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,2))

  
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//student_gam3_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NBias)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NBias) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Student")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,3))




#---------------------------------------------------------------------------------------------------
#
#  Figure 3 :    
#                            
#---------------------------------------------------------------------------------------------------

  dev.new()

#require(grid)
grid.newpage()
pushViewport(viewport(layout = grid.layout(2,3)))

define_region = function(row, col){
	viewport(layout.pos.row=row, layout.pos.col=col)
}


  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//frechet_gam2.5_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NMSE)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NMSE) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
#scale_y_continuous("MSE") +
scale_y_continuous("MSE", limit = c(0.06,0.61)) + # rho=2.5
#scale_y_continuous("MSE", limit = c(0.19,0.53)) + # rho=3
scale_x_continuous("k")+ # rho=4
labs(title = "Fréchet")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),
  vp=define_region(1,1))
   
  
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//pareto_gam2.5_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NMSE)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NMSE) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("MSE", limit = c(0.4,1.5)) + # rho=2.5
#scale_y_continuous("MSE") +
scale_x_continuous("k")+
labs(title = "Pareto")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,2))

    
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//student_gam2.5_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NMSE)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NMSE) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("MSE", limit = c(0,4)) + # rho=2.5
#scale_y_continuous("MSE") +
scale_x_continuous("k")+
labs(title = "Student")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(1,3))

   
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//frechet_gam2.5_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NBias)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NBias) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("BIAS", limit = c(0,1.2)) +
scale_x_continuous("k")+
labs(title = "Fréchet")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,1))


  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//pareto_gam2.5_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NBias)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NBias) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("BIAS") +
scale_x_continuous("k")+
labs(title = "Pareto")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,2))

  
  load("/Users/daouia/Desktop/Daouia_Gijbels_Stupfler_JASA/Code_Data_jasa/code_R_figures123//student_gam2.5_xtremiles.RData")
  
df_xt_als  <- data.frame(xxc = kk, yyc = xt_als_NBias)              
    df_xt  <- data.frame(xxc = kk, yyc = xt_NBias) 

dd <- ggplot(data=df_xt_als, aes(xxc, yyc)) + geom_line(size = .5, linetype=1) 
print(dd+geom_line(size = .5, data = df_xt, colour = "black", linetype=2) +
scale_y_continuous("BIAS", limit = c(0,1.7)) +
scale_x_continuous("k")+
labs(title = "Student")+ 
theme_bw() + theme(panel.border = element_blank(), panel.grid.major = element_blank(),
panel.grid.minor = element_blank(), axis.line = element_line(colour = "black")),       
  vp=define_region(2,3))







